# -*- coding: utf-8 -*-
import hashlib
from urllib.parse import urljoin

import scrapy
import time
from scrapy.spidermiddlewares.httperror import HttpError
from twisted.internet.error import DNSLookupError, TCPTimedOutError

from SCWSpider.items import ScwspiderItem


class ScwvehiclespiderSpider(scrapy.Spider):
    name = 'SCWVehicleSpider'
    # allowed_domains = ['https://www.cn357.com/']
    # start_urls = ['https://www.cn357.com/notice61794_BJ5059VBBEAG1']
    PAGE_URL = 'https://www.cn357.com/notice_{0}'
    DETAIL_URL = 'https://www.cn357.com/'
    item_name_list = {
        'vehicle_model': '//div/table/tr[2]/td[2]',
        'vehicle_batch': '//div/table/tr[2]/td[4]',
        'vehicle_brand': '//div/table/tr[3]/td[2]',
        'vehicle_type': '//div/table/tr[3]/td[4]',
        'approved_load': '//div/table/tr[4]/td[2]',
        'gross_mass': '//div/table/tr[4]/td[4]',
        'unladen_mass': '//div/table/tr[5]/td[2]',
        'fuel_type': '//div/table/tr[5]/td[4]',
        'emission_standard': '//div/table/tr[6]/td[2]',
        'axle_number': '//div/table/tr[6]/td[4]',
        'wheelbase': '//div/table/tr[7]/td[2]',
        'axle_load': '//div/table/tr[7]/td[4]',
        'spring_nums': '//div/table/tr[8]/td[2]',
        'tire_nums': '//div/table/tr[8]/td[4]',
        'tire_specification': '//div/table/tr[9]/td[2]',
        'approaching_departure_angle': '//div/table/tr[9]/td[4]',
        'front_rear_suspension': '//div/table/tr[10]/td[2]',
        'front_gauge': '//div/table/tr[10]/td[4]',
        'back_gauge': '//div/table/tr[11]/td[2]',
        'vehicle_id_number': '//div/table/tr[11]/td[4]',
        'vehicle_length': '//div/table/tr[12]/td[2]',
        'vehicle_width': '//div/table/tr[12]/td[4]',
        'vehicle_high': '//div/table/tr[13]/td[2]',
        'container_length': '//div/table/tr[13]/td[4]',
        'container_width': '//div/table/tr[14]/td[2]',
        'container_high': '//div/table/tr[14]/td[4]',
        'maximum_speed': '//div/table/tr[15]/td[2]',
        'seat': '//div/table/tr[15]/td[4]',
        'cab_nums': '//div/table/tr[16]/td[2]',
        'streering': '//div/table/tr[16]/td[4]',
        'allow_trailer_weight': '//div/table/tr[17]/td[2]',
        'zzllyxs': '//div/table/tr[17]/td[4]',
        'bgcazzdczzl': '//div/table/tr[18]/td[2]',
        'company_name': '//div/table/tr[18]/td[4]',
        'company_address': '//div/table/tr[19]/td[2]',
        'postcode': '//div/table/tr[20]/td[4]',
        'chasis1': '//div/table/tr[21]/td[2]',
        'chasis2': '//div/table/tr[21]/td[4]',
        'chasis3': '//div/table/tr[22]/td[2]',
        'chasis4': '//div/table/tr[22]/td[4]',
        'engine_info': '//div[@class="gMain"]/table/tr',
        'style_img_url': '//*[@id="noticeImage"]/ul/li',
    }

    def start_requests(self):
        for i in range(1, 353):
            # 第一页
            yield scrapy.Request(self.PAGE_URL.format(i), callback=self.parse, errback=self.errback_httpbin)

    def parse(self, response):
        self.log("===========================| %s |" % response.url)
        vehicle_model_list = response.xpath('/html/body/div[6]/div[1]/table/tr/td[1]/a')
        for vehicle_model in vehicle_model_list:
            url = urljoin(self.DETAIL_URL, vehicle_model.xpath('@href').extract()[0])
            self.log('vehilce_detail_url: {0}'.format(url))
            yield scrapy.Request(url, callback=self.parse_item)

        next_page = response.xpath('//span[@class="pageList"]/a')
        if next_page:
            next_page = next_page[-1]
            if next_page.xpath('text()').extract()[0] == '下一页':
                next_page = urljoin(self.PAGE_URL, next_page.xpath('@href').extract()[0])
                self.log('page_url: %s' % next_page)
                yield scrapy.Request(next_page, callback=self.parse)

    def parse_item(self, response):
        item = ScwspiderItem()

        for k in self.item_name_list.keys():
            item[k] = self.get_vehicle_info(response, k)

        self.log("===========================| {0} |".format(item['vehicle_model'] + item['vehicle_batch']))

        item['ut_url'] = response.url # 爬取url
        item['ut_type'] = 'scw_basic'  # 数据类型 scw_basic
        item['ut_source'] = '商车网'  # 数据来源 商车网
        item['ut_uptime'] = int(time.time())  # 获取时间戳
        item['ut_dotime'] = time.strftime("%Y-%m-%d") # 获取日期

        item['ut_tale'] = 'basic_vehicle'

        # item['ut_unique_id'] = scrapy.Field()  # 唯一性md5
        #hashlib.md5('你好'.encode(encoding='UTF-8')).hexdigest()
        yield item

    # 异常处理
    def errback_httpbin(self, failure):
        if failure.check(HttpError):
            response = failure.value.response
            print('HttpError on {0}'.format(response.url))

        elif failure.check(DNSLookupError):
            request = failure.request
            print('DNSLookupError on {0}'.format(request.url))

        elif failure.check(TimeoutError, TCPTimedOutError):
            request = failure.request
            print('TimeoutError on {0}'.format(request.url))


    def get_engine(self, response, field_name):
        engine_td = response.xpath(self.item_name_list[field_name])[-2].xpath('td[1]/table/tr[2]/td')
        if not engine_td:
            return []
        engine_models = engine_td[0].xpath('text()').extract()
        engine_companys = engine_td[1].xpath('text()').extract()
        engine_brands = engine_td[2].xpath('text()').extract()
        engine_ccs = engine_td[3].xpath('text()').extract()
        engine_powers = engine_td[4].xpath('text()').extract()

        engine_info = []
        for i in range(len(engine_models)):
            info = {}
            info['engine_model'] = engine_models[i] if len(engine_models) > i else ''
            info['engine_company'] = engine_companys[i] if len(engine_companys) > i else ''
            info['engine_brand'] = engine_brands[i] if len(engine_brands) > i else ''
            info['engine_cc'] = engine_ccs[i] if len(engine_ccs) > i else ''
            info['engine_power'] = engine_powers[i] if len(engine_powers) > i else ''
            engine_info.append(info)
        return engine_info

    def get_vehicle_info(self, response, field_name):
        if field_name == 'engine_info':
            return self.get_engine(response, field_name)
        if field_name == 'style_img_url':
            img_list = response.xpath(self.item_name_list[field_name])
            imgs = []
            if img_list:
                for i in img_list:
                    imgs.append(i.xpath('img').attrib.get('src'))
            return imgs
        field_list = response.xpath(self.item_name_list[field_name] + '//text()').extract()
        if field_list:
            field = ''
            for fd in field_list:
                field += fd
        else:
            field = ''
        return field

